#!/usr/bin/env node
'use strict';

const args = require('minimist')(process.argv.slice(2));
const lineReader = require('line-reader');
const fs = require('fs');
const colors = require('colors');

console.log(" Build stats generator - (c) Ikalogic SAS ".bgBrightBlue.white);
console.log("Usage: c2cpp -s my_c_file.h -d new_dest.cpp -r MAX1483_ -n my_namespace".bgBlue.white);

if ((typeof args.s != "string") || (typeof args.d != "string"))
{
    console.log("Invalid source and/or dest file name".red);
    process.exit(0);
}

if (typeof args.r != "string")
{
    args.r = "";
}
else
{
    args.r = args.r.toLowerCase();
}

//check if file exists.
fs.stat(args.s,function (err,stats){
    if(!stats)
    {
        console.log("file does not exists, aborting!".red);
        process.exit(0);
    }
});

var last_build;
var in_enum_group = false;
const regex_space = /[ \t]+/g;
const regex_comment = /^\/\//g;
const regex_line_end = /$/g;
const regex_special_chars = /[().#+-,:*%;!$]/g;
var ident_counter = 0;
var writer = fs.createWriteStream(args.d);
writer.write("//Automatically generated file from " + args.s + "\n");
if (typeof args.n == "string")
{
    writer.write("namespace " + args.n + "\n{\n");
    ident_counter++;
}
lineReader.eachLine(args.s,function(line,last){
    console.log(line.white);
    line = line.toLowerCase();
    if (line.includes("#endif"))
    {
        line = "";
        if (!last)return;
    }
    if (!last && (line.trim().length < 2)) return;

    if (!last)
    {
        if (line.search(regex_comment) >= 0)
        {
            if (in_enum_group)
            {
                ident_counter--;
                push_line("};\n");
            }
            console.log("ident="+ident_counter + " line = "+ line);
            line = "enum " + line.replace(regex_comment,"").replace(" ","_") + " {\n"
            in_enum_group = true;
            push_line(line);
            ident_counter++;
            return;
        }
        else
        {
            line = line.replace("#define","");
            if (line.search(regex_special_chars) >= 0)
            {
                line = "// " + line + " // <- invalid characters";
            }
            line = line.replace(args.r,"");
            line = line.trim();
            line = line.replace(regex_space, ' = ');
            line = line.replace(regex_line_end, ',');
        }
    }

    push_line(line);
    if (last)
    {
        console.log("Last".red);
        while (ident_counter)
        {
            ident_counter--;
            push_line("};\n");
        }
        writer.close();
    }
});

function push_line(line)
{
    for (var i = 0; i < ident_counter; i++)
    {
        line = "\t" + line;
    }
    console.log(line);
    writer.write(line + "\n");
}
